@@ -1,5 +1,6 @@
 cnmlcd = function(x, lcd, maxit=100, tol=1e-6,
-                  plot=c("null","density","logdensity","gradient")) {
+                  plot=c("null","density","logdensity","gradient"),
+                  verbose=0) {
   plot = match.arg(plot)
   lambda = 1e-15             # for psuedo-rank
   n = length(x)              # sample size
@@ -17,6 +18,14 @@ cnmlcd = function(x, lcd, maxit=100, tol=1e-6,
   attr(x, "xx") =                 # - colSums(pmax(outer(x,x,"-"),0) * w)
     rev(cumsum(rev(w))) * x - rev(cumsum(rev(x * w)))
   if(missing(lcd)) lcd = new.lcd(alpha=0, lower=lower, upper=upper)
+  else {
+    j = lcd$theta >= lower & lcd$theta <= upper
+    jx = indx(lcd$theta[j], x)
+    theta = x[jx]
+    jt = duplicated(theta)
+    lcd = new.lcd(alpha=lcd$alpha, theta=theta[!jt], pi=lcd$pi[j][!jt],
+                  lower=lower, upper=upper)
+  }
   ll = logLik.lcd(lcd, x, w)
   convergence = 1
   ll.old = -Inf
@@ -24,12 +33,9 @@ cnmlcd = function(x, lcd, maxit=100, tol=1e-6,
     if(ll <= ll.old  + tol) {convergence = 0; break}
     lcd.old = lcd
     ll.old = ll
-    switch(plot,
-           density = plot(lcd, x, w),
-           logdensity = plot(lcd, x, w, log=TRUE),
-           gradient = plotgradient(lcd, x, w),
-           null =, )
+    if(plot != "null") plot(lcd, x, w, type=plot) 
     g = maxima.gradient(lcd, x, w=w)      # finding new knots
+    if(verbose > 0) print.cnmlcdverb(verbose, lcd, ll, g$gmax, i)
     if(length(g$theta) != 0) {
       nsp = g$theta
       nsl = length(nsp)
@@ -62,39 +68,45 @@ cnmlcd = function(x, lcd, maxit=100, tol=1e-6,
     lcd1$alpha = -r1$x[1]       # lcd1$C will be updated inside line.lcd
     lcd1$pi = r1$x[-1]
     r = line.lcd(lcd, lcd1, x, w=w, ll0=ll.old)
-    lcd = r$lcd
-    ll = r$ll
-    ## if(any(j0 <- lcd$pi == 0)) {
-    ##   j = ! j0
-    ##   lcd = new.lcd(lcd$alpha, lcd$theta[j], lcd$pi[j], lcd$lower, lcd$upper)
-    ## }
-    if(any(lcd$pi == 0)) lcd = simplify.lcd(lcd)
+    if(r$convergence > 0) {
+      lcd = lcd.old; ll = ll.old
+      convergence = r$convergence + 1
+      break
+    }
+    else {
+      lcd = r$lcd; ll = r$ll
+      if(any(lcd$pi == 0)) lcd = simplify.lcd(lcd)
+    }
   }
-  list(lcd=lcd, ll=ll, num.iterations=i, max.gradient=g$gmax,
-       convergence=convergence)
+  structure(list(lcd=lcd, ll=ll, num.iterations=i, max.gradient=g$gmax,
+                 convergence=convergence),
+            class="cnmlcd")
 }
 
 ## Line search
 
+##### OUTPUT: 
+## convergence     = 0, success.
+##                 = 1, tolerance reached.
+##                 = 2, not an ascent direction.
+
 line.lcd = function(lcd0, lcd1, x, w, ll0, tol=1e-10) {
-  llt = function(alpha) {
-    m = new.lcd((1 - alpha) * lcd0$alpha + alpha * lcd1$alpha, lcd1$theta,
-        (1 - alpha) * lcd0$pi + alpha * lcd1$pi, lcd0$lower, lcd0$upper)
-    ll = logLik.lcd(m, x, w)
-    list(lcd=m, ll=ll)
-  }
   grad = gradient.lcd(lcd0, x, w, "knots")
   grad[1] = - grad[1]
   delta = sum(grad * (c(lcd1$alpha, lcd1$pi) - c(lcd0$alpha, lcd0$pi))) * .333
+  if(delta <= 0) return(list(lcd=lcd0, ll=ll0, alpha=0, convergence=2))
   convergence = 0
   alpha = 1
   repeat{
-    new = llt(alpha)
-    if(new$ll >= ll0 + alpha * delta) break 
-    if(alpha < tol) {convergence=1; new=list(lcd=lcd0, ll=ll0); break}
+    lcdt = new.lcd((1 - alpha) * lcd0$alpha + alpha * lcd1$alpha, lcd1$theta,
+                   (1 - alpha) * lcd0$pi + alpha * lcd1$pi,
+                   lcd0$lower, lcd0$upper)
+    llt = logLik.lcd(lcdt, x, w)
+    if(llt >= ll0 + alpha * delta) break 
+    if(alpha < tol) {convergence=1; lcdt=lcd0; llt=ll0; break}
     alpha = alpha * .5
   }
-  list(lcd=new$lcd, ll=new$ll, alpha=alpha, convergence=convergence)
+  list(lcd=lcdt, ll=llt, alpha=alpha, convergence=convergence)
 }
 
 ## Create a new 'lcd' object
@@ -113,7 +125,7 @@ line.lcd = function(lcd0, lcd1, x, w, ll0, tol=1e-10) {
 ## lower      Lower boundary (= x_1)
 ## upper      Upper boundary (= x_n)
 ## coef       Intercepts (in row 1) and slopes (in row 2) over all segments
-## fk         Density at knots = c(lower, theta)
+## fk         Density values at knots = c(lower, theta)
 ## dpk        Int x^o f(x) dx over each segment between knots for o = 0, 1, 2
 ## cpk        Cumulative sum of dpk for all segments
 
@@ -159,18 +171,18 @@ new.lcd = function(alpha, theta=NULL, pi=NULL, lower, upper) {
 }
 
 simplify.lcd = function(lcd) {
-  if(any(j0 <- lcd$pi == 0)) {
+  if(any(j <- lcd$pi == 0)) {
     nk = length(lcd$theta) + 1
-    j = which(!j0)
-    pi = lcd$pi[j]
-    theta = lcd$theta[j]
-    j1 = c(1,j+1)
-    dpk = cpk = lcd$cpk[, c(j,nk), drop=FALSE]
-    for(i in 1:nrow(dpk)) dpk[i,] = c(cpk[i,1], diff(cpk[i,]))
+    jk = which(jn <- ! j)
+    pi = lcd$pi[jk]
+    theta = lcd$theta[jk]
+    jk1 = c(1,jk+1)
+    dpk = cpk = lcd$cpk[, c(jk,nk), drop=FALSE]
+    for(i in 1:nrow(dpk)) dpk[i,] = tapply(lcd$dpk[i,], cumsum(c(1, jn)), sum)
     lcd = structure(list(alpha=lcd$alpha, C=lcd$C, theta=theta, pi=pi,
                          lower=lcd$lower, upper=lcd$upper,
-                         coef=lcd$coef[,j1,drop=FALSE],
-                         fk=lcd$fk[j1], dpk=dpk, cpk=cpk),
+                         coef=lcd$coef[,jk1,drop=FALSE],
+                         fk=lcd$fk[jk1], dpk=dpk, cpk=cpk),
                     class = "lcd")
   }
   lcd
@@ -211,13 +223,16 @@ plcd = function(q, lcd, lower.tail=TRUE, log.p=FALSE) {
 maxima.gradient = function(lcd, x, w, tol = -Inf) {
   knots = c(lcd$lower, lcd$theta, lcd$upper)
   nk = length(knots)
-  index = match(knots, x)
   grad = gradient.lcd(lcd, x, w, "x")
   ii = integer(nk - 1)
-  for(i in 1:(nk-1)) {
+  index = indx(knots, x)
+  for(i in 1:(nk-1)) {        
     ima = which.max(grad[index[i]:index[i+1]])
     ii[i] = index[i] + ima - 1
   }
+##   index = indx(x, c(lcd$lower, lcd$theta))       # use tapply()?
+##   ima = tapply(grad, index, which.max)           #  right knots not included
+##   ii = c(1, which(diff(index) == 1)) + ima - 1   # not as good
   theta = x[ii]
   g = grad[ii]
   gmax = max(g)         # maximum gradient over {x_1, ..., x_n}
@@ -292,24 +307,20 @@ cpx = function(lcd, x, order=0) {
   else dpx 
 }
 
-## Plot the gradient function
-
-plotgradient = function(lcd, x, w=NULL, col="blue", knotcol=col, pch=1,
-                        lwd=1, lty=1, ...) {
-  if(is.null(w)) { xw = x.weight(x); x = xw$x; w = xw$w }
-  y = gradient.lcd(lcd, x, w, "x")
-  plot(0, 0, xlim=range(x), ylim=range(y, 0), type="n",
-       ylab="Gradient", xlab=expression(theta), ...)
-  abline(h=0, col='lightgrey', lty=1)
-  lines(x, y, col=col, lwd=lwd)
-  knots = c(lcd$lower, lcd$theta, lcd$upper)
-  points(knots, y[indx(knots, x)], col=knotcol, pch=pch)
-}
+plot.cnmlcd = function(x, ...) plot(x$lcd, ...)
 
 ## plot density or log-density function
 
-plot.lcd = function(x, data, w=NULL, log=FALSE, col="blue", knotcol=col,
-                    border="grey", lwd=1, pch=1, main, breaks=50, ...) {
+plot.lcd = function(x, data, w=NULL, type=c("density","logdensity","gradient"),
+                    col="blue", knotcol=col, border="grey",
+                    lwd=1, lty=1, pch=1, main, breaks=50, ...) {
+  type = match.arg(type)
+  if(type == "gradient") {
+    plotgradient(x, data, w=w, col=col, knotcol=knotcol, lwd=lwd,
+                 lty=lty, pch=pch, main=main)
+    return(invisible())
+  }
+  log = type == "logdensity" 
   if(!is.null(w)) data = rep(data, w)
   p = sort( c(seq(x$lower, x$upper, len=200), x$theta) )
   sp = c(x$lower, x$theta, x$upper)
@@ -329,7 +340,7 @@ plot.lcd = function(x, data, w=NULL, log=FALSE, col="blue", knotcol=col,
     ylim = c(min(d), max(d))
     if(missing(data)) {
       plot(p, d, xlab="Data", ylab="Log-density", ylim=ylim,
-           main=main, type="l", lwd=lwd, col=col, ...)
+           main=main, type="l", lwd=lwd, col=col, lty=lty, ...)
       base = min(d)
     }
     else{
@@ -341,7 +352,7 @@ plot.lcd = function(x, data, w=NULL, log=FALSE, col="blue", knotcol=col,
       if(missing(main)) main =  "Log density estimation"
       loghist(data, h=h, main=main, ylim=ylim,
               xlab="Data", base=base, border=border, ...)
-      lines(p, d, col=col, lwd=lwd)
+      lines(p, d, col=col, lwd=lwd, lty=lty)
     }
     segments(p[1], base, p[1], d[1], col=col, lwd=lwd)
     segments(p[np], base, p[np], d[np], col=col, lwd=lwd)
@@ -352,12 +363,11 @@ plot.lcd = function(x, data, w=NULL, log=FALSE, col="blue", knotcol=col,
     if(!missing(data)) {
       hist(data, breaks=breaks, freq=F, ylim=ylim, xlab="Data",
            main=main, border=border, ...)
-      lines(p, d, col=col, lty=1, lwd=lwd)
+      lines(p, d, col=col, lwd=lwd, lty=lty)
     }
     else {
-      plot(p, d, col=col, type="l", lwd=lwd,
-           xlab="Data", ylab="Density", 
-           main=main, ...)
+      plot(p, d, col=col, type="l", lwd=lwd, lty=lty,
+           xlab="Data", ylab="Density", main=main, ...)
     }
     segments(p[1], 0, p[1], d[1], col=col, lwd=lwd)
     segments(p[np], 0, p[np], d[np], col=col, lwd=lwd)
@@ -365,6 +375,21 @@ plot.lcd = function(x, data, w=NULL, log=FALSE, col="blue", knotcol=col,
   }
 }
 
+## Plot the gradient function
+
+plotgradient = function(lcd, x, w=NULL, col="blue", knotcol=col, 
+                        lwd=1, lty=1, pch=1, main, ...) {
+  if(is.null(w)) { xw = x.weight(x); x = xw$x; w = xw$w }
+  y = gradient.lcd(lcd, x, w, "x")
+  if(missing(main)) main = ""
+  plot(0, 0, xlim=range(x), ylim=range(y, 0), type="n",
+       ylab="Gradient", xlab=expression(theta), main=main, ...)
+  abline(h=0, col='lightgrey', lty=1)
+  lines(x, y, col=col, lwd=lwd)
+  knots = c(lcd$lower, lcd$theta, lcd$upper)
+  points(knots, y[indx(knots, x)], col=knotcol, pch=pch)
+}
+
 loghist = function(x, h=NULL, breaks=30, main=NULL, ylim=NULL,
     xlim=NULL, base=NULL, xlab="x", ylab="Log-density", lty=1, border=1) {
   if(is.null(h)) h = hist(x, breaks=breaks, plot=FALSE)
@@ -384,6 +409,14 @@ loghist = function(x, h=NULL, breaks=30, main=NULL, ylim=NULL,
            lty=lty, col=border)                   # vertical
 }
 
+print.cnmlcdverb = function(verbose=0, lcd, ll, gmax, i) {
+  if(verbose > 0)
+    cat("## Iteration ", i, ":\n   Log-likelihood = ", as.character(ll), "\n", sep="")
+  if(verbose > 1) cat("   Max gradient =", gmax, "\n")
+  if(verbose == 3) {cat("   Interior knots =\n"); print(lcd$theta)}
+  if(verbose > 3) {cat("   lcd =\n"); print(lcd) }
+}
+
 ## Print an 'lcd' object
 
 print.lcd = function(x, ...) {
